function H=QubitHamiltonian(Ej,Ec,alpha,f,nmax,ng1,ng2)
% H=QubitHamiltonian(Ej,Ec,alpha,f,nmax,ng1,ng2)
% Qubit Hamiltonian in the charge base
%    Ej - Josephson energy E_j=\frac{I_c \phi_0}{2 \pi}
%    Ec - charging energy E_c=\frac{e^2}{2 C}
%    alpha - relative size of the top junction compared to the left and right junctions
%    f - frustration f=\frac{B A}{\phi_0}
%    nmax - maximum number of cooper pairs on an island - charge states run from -nmax to nmax
%    ng1 - gate charge induced on island 1 (top left island)
%    ng2 - gate charge induced on island 2 (top left island)

% QubitHamiltonian 1.1  11.1.2002  Hannes Majer (majer@qt.tn.tudelft.nl)


if nargin~=7 
   error('Incorrect number of input arguments.');
end


N = 2*nmax+1;  % number of charge states

[R,Q]=meshgrid(1:N^2,1:N^2);

Qp=Q-(N+1)^2/2;
Rp=R-(N+1)^2/2;

n1=round((Qp)/N)+1;   %formula (28)
n2=Qp-(n1-1)*N;
m1=round((Rp)/N)+1;
m2=Rp-(m1-1)*N;

H=-Ej/2*(abs(m1-n1)==1).*(m2==n2);  % formula (25)
H=H-Ej/2*(abs(m2-n2)==1).*(m1==n1); % formula (26)
H=H-alpha*Ej/2*((m1==n1+1).*(m2==n2-1)*exp(-j*2*pi*f)+(m1==n1-1).*(m2==n2+1)*exp(j*2*pi*f)); % formula (27)

n1=diag(ones(N^2,1)).*n1;
n2=diag(ones(N^2,1)).*n2;

ng1=diag(ones(N^2,1)).*ng1;
ng2=diag(ones(N^2,1)).*ng2;

n3=-n1-n2;
H=H + 4*Ec*((1+alpha)/(1+2*alpha))*(n1-ng1).^2 + 4*Ec*((2*alpha)/(1+2*alpha))*(n1-ng1).*(n2-ng2) + 4*Ec*((1+alpha)/(1+2*alpha))*(n2-ng2).^2; % fomula (17)

